@@ -1,29 +1,36 @@
-/*
- * Copyright © 2002 Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
- * California 95054, U.S.A. All rights reserved.  Sun Microsystems, Inc. has
- * intellectual property rights relating to technology embodied in the product
- * that is described in this document. In particular, and without limitation,
- * these intellectual property rights may include one or more of the U.S.
- * patents listed at http://www.sun.com/patents and one or more additional
- * patents or pending patent applications in the U.S. and in other countries.
- * U.S. Government Rights - Commercial software. Government users are subject
- * to the Sun Microsystems, Inc. standard license agreement and applicable
- * provisions of the FAR and its supplements.  Use is subject to license terms.
- * Sun,  Sun Microsystems,  the Sun logo and  Java are trademarks or registered
- * trademarks of Sun Microsystems, Inc. in the U.S. and other countries.  This
- * product is covered and controlled by U.S. Export Control laws and may be
- * subject to the export or import laws in other countries.  Nuclear, missile,
- * chemical biological weapons or nuclear maritime end uses or end users,
- * whether direct or indirect, are strictly prohibited.  Export or reexport
- * to countries subject to U.S. embargo or to entities identified on U.S.
- * export exclusion lists, including, but not limited to, the denied persons
- * and specially designated nationals lists is strictly prohibited.
+/* Copyright (c) 2006, Sun Microsystems, Inc.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *     * Redistributions of source code must retain the above copyright notice,
+ *       this list of conditions and the following disclaimer.
+ *     * Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer in the
+ *       documentation and/or other materials provided with the distribution.
+ *     * Neither the name of the Sun Microsystems, Inc. nor the names of its
+ *       contributors may be used to endorse or promote products derived from
+ *       this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
  */
-
 package org.javacc.parser;
 
-import java.util.Vector;
-import java.io.*;
+import java.io.File;
+import java.util.ArrayList;
+import java.util.List;
+
 import org.javacc.Version;
 
 /**
@@ -44,47 +51,32 @@ public class JavaCCGlobals {
   static public String fileName;
 
   /**
-   * The name of the original file (before processing by JJTree and JJCov).
+   * The name of the original file (before processing by JJTree).
    * Currently this is the same as fileName.
    */
   static public String origFileName;
 
-  /**
-   * The File object for the output directory where all the generated files will
-   * be placed. It defaults to the current directory, but may be set by the
-   * user using the OUTPUT_DIRECTORY option.
-   */
-  static public File outputDir = new File(System.getProperty("user.dir"));
-  static private String outputDirName = null;
-  static private Object outputDirNameLoc = null;
-
   /**
    * Set to true if this file has been processed by JJTree.
    */
   static public boolean jjtreeGenerated;
 
-  /**
-   * Set to true if this file has been processed by JJCov.
-   */
-  static public boolean jjcovGenerated;
-
   /**
    * The list of tools that have participated in generating the
    * input grammar file.
    */
-  static public Vector toolNames;
+  static public List toolNames;
 
   /**
    * This prints the banner line when the various tools are invoked.  This
    * takes as argument the tool's full name and its version.
    */
   static public void bannerLine(String fullName, String ver) {
-    System.out.print("Java Compiler Compiler Version " + Version.version + " (" + fullName);
+    System.out.print("Java Compiler Compiler Version " + Version.versionNumber + " (" + fullName);
     if (!ver.equals("")) {
       System.out.print(" Version " + ver);
     }
     System.out.println(")");
-    //System.out.println("Copyright © 2002 Sun Microsystems, Inc. All rights reserved. Use is subject to license terms.");
   }
 
   /**
@@ -96,34 +88,34 @@ public class JavaCCGlobals {
    * This is a list of tokens that appear after "PARSER_BEGIN(name)" all the
    * way until (but not including) the opening brace "{" of the class "name".
    */
-  static public java.util.Vector cu_to_insertion_point_1 = new java.util.Vector();
+  static public java.util.List cu_to_insertion_point_1 = new java.util.ArrayList();
 
   /**
    * This is the list of all tokens that appear after the tokens in
    * "cu_to_insertion_point_1" and until (but not including) the closing brace "}"
    * of the class "name".
    */
-  static public java.util.Vector cu_to_insertion_point_2 = new java.util.Vector();
+  static public java.util.List cu_to_insertion_point_2 = new java.util.ArrayList();
 
   /**
    * This is the list of all tokens that appear after the tokens in
    * "cu_to_insertion_point_2" and until "PARSER_END(name)".
    */
-  static public java.util.Vector cu_from_insertion_point_2 = new java.util.Vector();
+  static public java.util.List cu_from_insertion_point_2 = new java.util.ArrayList();
 
   /**
    * A list of all grammar productions - normal and JAVACODE - in the order
    * they appear in the input file.  Each entry here will be a subclass of
    * "NormalProduction".
    */
-  static public java.util.Vector bnfproductions = new java.util.Vector();
+  static public java.util.List bnfproductions = new java.util.ArrayList();
 
   /**
    * A symbol table of all grammar productions - normal and JAVACODE.  The
    * symbol table is indexed by the name of the left hand side non-terminal.
    * Its contents are of type "NormalProduction".
    */
-  static public java.util.Hashtable production_table = new java.util.Hashtable();
+  static public java.util.Map production_table = new java.util.HashMap();
 
   /**
    * A mapping of lexical state strings to their integer internal representation.
@@ -140,14 +132,14 @@ public class JavaCCGlobals {
   /**
    * The declarations to be inserted into the TokenManager class.
    */
-  static public java.util.Vector token_mgr_decls;
+  static public java.util.List token_mgr_decls;
 
   /**
    * The list of all TokenProductions from the input file.  This list includes
    * implicit TokenProductions that are created for uses of regular expressions
    * within BNF productions.
    */
-  static public java.util.Vector rexprlist = new java.util.Vector();
+  static public java.util.List rexprlist = new java.util.ArrayList();
 
   /**
    * The total number of distinct tokens.  This is therefore one more than the
@@ -160,13 +152,13 @@ public class JavaCCGlobals {
    * defined with a label).  The index to the table is the image of the label
    * and the contents of the table are of type "RegularExpression".
    */
-  static public java.util.Hashtable named_tokens_table = new java.util.Hashtable();
+  static public java.util.Map named_tokens_table = new java.util.HashMap();
 
   /**
    * Contains the same entries as "named_tokens_table", but this is an ordered
    * list which is ordered by the order of appearance in the input file.
    */
-  static public java.util.Vector ordered_named_tokens = new java.util.Vector();
+  static public java.util.List ordered_named_tokens = new java.util.ArrayList();
 
   /**
    * A mapping of ordinal values (represented as objects of type "Integer") to
@@ -175,13 +167,13 @@ public class JavaCCGlobals {
    * If there are multiple labels representing the same ordinal value, then
    * only one label is stored.
    */
-  static public java.util.Hashtable names_of_tokens = new java.util.Hashtable();
+  static public java.util.Map names_of_tokens = new java.util.HashMap();
 
   /**
    * A mapping of ordinal values (represented as objects of type "Integer") to
    * the corresponding RegularExpression's.
    */
-  static public java.util.Hashtable rexps_of_tokens = new java.util.Hashtable();
+  static public java.util.Map rexps_of_tokens = new java.util.HashMap();
 
   /**
    * This is a three-level symbol table that contains all simple tokens (those
@@ -200,7 +192,8 @@ public class JavaCCGlobals {
    */
   static protected int maskindex = 0;
   static protected int jj2index = 0;
-  static protected Vector maskVals = new Vector();
+  public static boolean lookaheadNeeded;
+  static protected List maskVals = new ArrayList();
 
   static Action actForEof;
   static String nextStateForEof;
@@ -213,8 +206,8 @@ public class JavaCCGlobals {
    * used to generate it.
    */
   public static String getIdString(String toolName, String fileName) {
-     Vector toolNames = new Vector();
-     toolNames.addElement(toolName);
+     List toolNames = new ArrayList();
+     toolNames.add(toolName);
      return getIdString(toolNames, fileName);
   }
 
@@ -222,13 +215,13 @@ public class JavaCCGlobals {
    * Returns the identifying string for the file name, given a set of tool
    * names that are used to generate it.
    */
-  public static String getIdString(Vector toolNames, String fileName) {
+  public static String getIdString(List toolNames, String fileName) {
      int i;
      String toolNamePrefix = "Generated By:";
 
      for (i = 0; i < toolNames.size() - 1; i++)
-        toolNamePrefix += (String)toolNames.elementAt(i) + "&";
-     toolNamePrefix += (String)toolNames.elementAt(i) + ":";
+        toolNamePrefix += (String)toolNames.get(i) + "&";
+     toolNamePrefix += (String)toolNames.get(i) + ":";
 
      if (toolNamePrefix.length() > 200)
      {
@@ -236,7 +229,7 @@ public class JavaCCGlobals {
         throw new Error();
      }
 
-     return toolNamePrefix + " Do not edit this line. " + fileName;
+     return toolNamePrefix + " Do not edit this line. " + addUnicodeEscapes(fileName);
   }
 
   /**
@@ -244,17 +237,17 @@ public class JavaCCGlobals {
    * by getToolNames(fileName).
    */
   public static boolean isGeneratedBy(String toolName, String fileName) {
-     Vector v = getToolNames(fileName);
+     List v = getToolNames(fileName);
 
      for (int i = 0; i < v.size(); i++)
-        if (toolName.equals((String)v.elementAt(i)))
+        if (toolName.equals(v.get(i)))
            return true;
 
      return false;
   }
 
-  private static Vector makeToolNameVector(String str) {
-     Vector retVal = new Vector();
+  private static List makeToolNameList(String str) {
+     List retVal = new ArrayList();
 
      int limit1 = str.indexOf('\n');
      if (limit1 == -1) limit1 = 1000;
@@ -283,21 +276,21 @@ public class JavaCCGlobals {
 
      while (j < tmp.length() && (i = tmp.indexOf('&', j)) != -1)
      {
-        retVal.addElement(tmp.substring(j, i));
+        retVal.add(tmp.substring(j, i));
         j = i + 1;
      }
 
      if (j < tmp.length())
-        retVal.addElement(tmp.substring(j));
+        retVal.add(tmp.substring(j));
 
      return retVal;
   }
 
   /**
-   * Returns a Vector of names of the tools that have been used to generate
+   * Returns a List of names of the tools that have been used to generate
    * the given file.
    */
-  public static Vector getToolNames(String fileName) {
+  public static List getToolNames(String fileName) {
      char[] buf = new char[256];
      java.io.FileReader stream = null;
      int read, total = 0;
@@ -314,70 +307,44 @@ public class JavaCCGlobals {
           else
              break;
 
-       return makeToolNameVector(new String(buf, 0, total));
+       return makeToolNameList(new String(buf, 0, total));
     } catch(java.io.FileNotFoundException e1) {
     } catch(java.io.IOException e2) {
        if (total > 0)
-          return makeToolNameVector(new String(buf, 0, total));
+          return makeToolNameList(new String(buf, 0, total));
     }
     finally {
-       try { stream.close(); }
-       catch (Exception e3) { }
+       if (stream != null)
+         try { stream.close(); }
+         catch (Exception e3) { }
     }
 
-    return new Vector();
+    return new ArrayList();
   }
 
-  public static void storeOutputDirSpec(Object loc, String val) {
-    outputDirName = val;
-    outputDirNameLoc = loc;
-  }
+  public static void createOutputDir(File outputDir) {
+    if (!outputDir.exists()) {
+      JavaCCErrors.warning("Output directory \"" + outputDir + "\" does not exist. Creating the directory.");
 
-  public static void setOutputDir() {
-     if (outputDirName == null)
+      if (!outputDir.mkdirs()) {
+        JavaCCErrors.semantic_error("Cannot create the output directory : " + outputDir);
         return;
+      }
+    }
 
-     File tmp = new File(outputDirName);
+    if (!outputDir.isDirectory()) {
+      JavaCCErrors.semantic_error("\"" + outputDir + " is not a valid output directory.");
+      return;
+    }
 
-     if (!tmp.exists()) {
-        if (outputDirNameLoc == null)
-           JavaCCErrors.warning("Output directory \"" + outputDirName + "\" does not exist. Creating the directory.");
-        else
-           JavaCCErrors.warning(outputDirNameLoc, "Output directory \"" + outputDirName + "\" does not exist. Creating the directory.");
-
-        if (!tmp.mkdir()) {
-           if (outputDirNameLoc == null)
-              JavaCCErrors.semantic_error("Cannot create the output directory : " + outputDirName);
-           else
-              JavaCCErrors.semantic_error(outputDirNameLoc, "Cannot create the output directory : " + outputDirName);
-
-           return;
-        }
-     }
-
-     if (!tmp.isDirectory()) {
-        if (outputDirNameLoc == null)
-           JavaCCErrors.semantic_error("\"" + outputDirName + " is not a valid output directory.");
-        else
-           JavaCCErrors.semantic_error(outputDirNameLoc, "\"" + outputDirName + " is not a valid output directory.");
-
-        return;
-     }
-
-     if (!tmp.canWrite()) {
-        if (outputDirNameLoc == null)
-           JavaCCErrors.semantic_error("Cannot write to the output output directory : \"" + outputDirName + "\"");
-        else
-           JavaCCErrors.semantic_error(outputDirNameLoc, "Cannot write to the output output directory : \"" + outputDirName + "\"");
-
-        return;
-     }
-
-     outputDir = tmp;
+    if (!outputDir.canWrite()) {
+      JavaCCErrors.semantic_error("Cannot write to the output output directory : \"" + outputDir + "\"");
+      return;
+    }
   }
 
   static public String staticOpt() {
-    if (Options.B("STATIC")) {
+    if (Options.getStatic()) {
       return "static ";
     } else {
       return "";
@@ -406,8 +373,8 @@ public class JavaCCGlobals {
       } else if (ch == '\\') {
         retval += "\\\\";
       } else if (ch < 0x20 || ch > 0x7e) {
-	String s = "0000" + Integer.toString(ch, 16);
-	retval += "\\u" + s.substring(s.length() - 4, s.length());
+        String s = "0000" + Integer.toString(ch, 16);
+        retval += "\\u" + s.substring(s.length() - 4, s.length());
       } else {
         retval += ch;
       }
@@ -420,9 +387,9 @@ public class JavaCCGlobals {
     char ch;
     for (int i = 0; i < str.length(); i++) {
       ch = str.charAt(i);
-      if (ch < 0x20 || ch > 0x7e) {
-	String s = "0000" + Integer.toString(ch, 16);
-	retval += "\\u" + s.substring(s.length() - 4, s.length());
+      if (ch < 0x20 || ch > 0x7e || ch == '\\') {
+        String s = "0000" + Integer.toString(ch, 16);
+        retval += "\\u" + s.substring(s.length() - 4, s.length());
       } else {
         retval += ch;
       }
@@ -471,6 +438,17 @@ public class JavaCCGlobals {
     printTokenOnly(t, ostr);
   }
 
+  static protected void printTokenList(List list, java.io.PrintWriter ostr) {
+    Token t = null;
+    for (java.util.Iterator it = list.iterator(); it.hasNext();) {
+      t = (Token)it.next();
+      printToken(t, ostr);
+    }
+    
+    if (t != null)
+      printTrailingComments(t, ostr);
+  }
+
   static protected void printLeadingComments(Token t, java.io.PrintWriter ostr) {
     if (t.specialToken == null) return;
     Token tt = t.specialToken;
@@ -551,33 +529,29 @@ public class JavaCCGlobals {
    {
       fileName = null;
       origFileName = null;
-      outputDir = new File(System.getProperty("user.dir"));
-      outputDirName = null;
-      outputDirNameLoc = null;
       jjtreeGenerated = false;
-      jjcovGenerated = false;
       toolNames = null;
       cu_name = null;
-      cu_to_insertion_point_1 = new java.util.Vector();
-      cu_to_insertion_point_2 = new java.util.Vector();
-      cu_from_insertion_point_2 = new java.util.Vector();
-      bnfproductions = new java.util.Vector();
-      production_table = new java.util.Hashtable();
+      cu_to_insertion_point_1 = new java.util.ArrayList();
+      cu_to_insertion_point_2 = new java.util.ArrayList();
+      cu_from_insertion_point_2 = new java.util.ArrayList();
+      bnfproductions = new java.util.ArrayList();
+      production_table = new java.util.HashMap();
       lexstate_S2I = new java.util.Hashtable();
       lexstate_I2S = new java.util.Hashtable();
       token_mgr_decls = null;
-      rexprlist = new java.util.Vector();
-      tokenCount = (int)0;
-      named_tokens_table = new java.util.Hashtable();
-      ordered_named_tokens = new java.util.Vector();
-      names_of_tokens = new java.util.Hashtable();
-      rexps_of_tokens = new java.util.Hashtable();
+      rexprlist = new java.util.ArrayList();
+      tokenCount = 0;
+      named_tokens_table = new java.util.HashMap();
+      ordered_named_tokens = new java.util.ArrayList();
+      names_of_tokens = new java.util.HashMap();
+      rexps_of_tokens = new java.util.HashMap();
       simple_tokens_table = new java.util.Hashtable();
       maskindex = 0;
       jj2index = 0;
-      maskVals = new Vector();
-      cline = (int)0;
-      ccol = (int)0;
+      maskVals = new ArrayList();
+      cline = 0;
+      ccol = 0;
       actForEof = null;
       nextStateForEof = null;
    }
