@@ -1,28 +1,34 @@
-/*
- * Copyright © 2002 Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
- * California 95054, U.S.A. All rights reserved.  Sun Microsystems, Inc. has
- * intellectual property rights relating to technology embodied in the product
- * that is described in this document. In particular, and without limitation,
- * these intellectual property rights may include one or more of the U.S.
- * patents listed at http://www.sun.com/patents and one or more additional
- * patents or pending patent applications in the U.S. and in other countries.
- * U.S. Government Rights - Commercial software. Government users are subject
- * to the Sun Microsystems, Inc. standard license agreement and applicable
- * provisions of the FAR and its supplements.  Use is subject to license terms.
- * Sun,  Sun Microsystems,  the Sun logo and  Java are trademarks or registered
- * trademarks of Sun Microsystems, Inc. in the U.S. and other countries.  This
- * product is covered and controlled by U.S. Export Control laws and may be
- * subject to the export or import laws in other countries.  Nuclear, missile,
- * chemical biological weapons or nuclear maritime end uses or end users,
- * whether direct or indirect, are strictly prohibited.  Export or reexport
- * to countries subject to U.S. embargo or to entities identified on U.S.
- * export exclusion lists, including, but not limited to, the denied persons
- * and specially designated nationals lists is strictly prohibited.
+/* Copyright (c) 2006, Sun Microsystems, Inc.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *     * Redistributions of source code must retain the above copyright notice,
+ *       this list of conditions and the following disclaimer.
+ *     * Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer in the
+ *       documentation and/or other materials provided with the distribution.
+ *     * Neither the name of the Sun Microsystems, Inc. nor the names of its
+ *       contributors may be used to endorse or promote products derived from
+ *       this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
+ * THE POSSIBILITY OF SUCH DAMAGE.
  */
-
 package org.javacc.parser;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.List;
 
 /**
  * Describes character lists.
@@ -37,10 +43,10 @@ public class RCharacterList extends RegularExpression {
   public boolean negated_list = false;
 
   /**
-   * This is the list of descriptors of the character list.  Each vector
+   * This is the list of descriptors of the character list.  Each list
    * entry will narrow to either SingleCharacter or to CharacterRange.
    */
-  public java.util.Vector descriptors = new java.util.Vector();
+  public List descriptors = new ArrayList();
 
 static final char[] diffLowerCaseRanges = {
 65, 90, 192, 214, 216, 222, 256, 256, 258, 258, 260, 260, 262, 262, 264, 264,
@@ -52,7 +58,7 @@ static final char[] diffLowerCaseRanges = {
 334, 334, 336, 336, 338, 338, 340, 340, 342, 342, 344, 344, 346, 346, 348, 348,
 350, 350, 352, 352, 354, 354, 356, 356, 358, 358, 360, 360, 362, 362, 364, 364,
 366, 366, 368, 368, 370, 370, 372, 372, 374, 374, 376, 376, 377, 377, 379, 379,
-381, 381, 385, 385, 386, 386, 388, 388, 390, 390, 391, 391, 
+381, 381, 385, 385, 386, 386, 388, 388, 390, 390, 391, 391,
 /* new for fixing 1.0.2 */ 393, 393, /* End new */ 394, 394, 395, 395,
 /*398, Sreeni fixed for 1.2*/ 399, 399, 400, 400, 401, 401, 403, 403, 404, 404, 406, 406, 407, 407, 408, 408,
 412, 412, 413, 413, 416, 416, 418, 418, 420, 420, 423, 423, 425, 425, 428, 428,
@@ -119,7 +125,7 @@ static final char[] diffUpperCaseRanges = {
 476, 476, 479, 479, 481, 481, 483, 483, 485, 485, 487, 487, 489, 489, 491, 491,
 493, 493, 495, 495, 498, 498, 499, 499, 501, 501, 507, 507, 509, 509, 511, 511,
 513, 513, 515, 515, 517, 517, 519, 519, 521, 521, 523, 523, 525, 525, 527, 527,
-529, 529, 531, 531, 533, 533, 535, 535, 595, 595, 596, 596, 598, 
+529, 529, 531, 531, 533, 533, 535, 535, 595, 595, 596, 596, 598,
 /* new for fixing 1.0.2 */ 598, 599, /* End new */ 599, /*600, Sreeni fixed for 1.2 */
 601, 601,
 603, 603, 608, 608, 611, 611, 616, 616, 617, 617, 623, 623, 626, 626, 643, 643,
@@ -168,132 +174,131 @@ static final char[] diffUpperCaseRanges = {
 
   void ToCaseNeutral()
   {
-     int cnt = descriptors.size();
+    int cnt = descriptors.size();
 
-     OuterLoop:
-     for (int i = 0; i < cnt; i++)
-     {
-        if (descriptors.elementAt(i) instanceof SingleCharacter)
-        {
-           char ch = ((SingleCharacter)descriptors.elementAt(i)).ch;
+    for (int i = 0; i < cnt; i++)
+    {
+      if (descriptors.get(i) instanceof SingleCharacter)
+      {
+        char ch = ((SingleCharacter)descriptors.get(i)).ch;
 
-           if (ch != Character.toLowerCase(ch))
-              descriptors.addElement(new 
+        if (ch != Character.toLowerCase(ch))
+          descriptors.add(new
                          SingleCharacter(Character.toLowerCase(ch)));
-           if (ch != Character.toUpperCase(ch))
-              descriptors.addElement(new 
+        if (ch != Character.toUpperCase(ch))
+           descriptors.add(new
                          SingleCharacter(Character.toUpperCase(ch)));
+      }
+      else
+      {
+        char l = ((CharacterRange)descriptors.get(i)).getLeft();
+        char r = ((CharacterRange)descriptors.get(i)).getRight();
+        int j = 0;
+
+        /* Add ranges for which lower case is different. */
+        for (;;)
+        {
+          while (l > diffLowerCaseRanges[j])
+            j += 2;
+
+          if (l < diffLowerCaseRanges[j])
+          {
+            if (r < diffLowerCaseRanges[j])
+              break;
+
+            if (r <= diffLowerCaseRanges[j + 1])
+            {
+              descriptors.add(new CharacterRange(Character.toLowerCase(diffLowerCaseRanges[j]),
+                   (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + r - diffLowerCaseRanges[j])));
+              break;
+            }
+
+            descriptors.add(new CharacterRange(Character.toLowerCase(diffLowerCaseRanges[j]),
+                                                       Character.toLowerCase(diffLowerCaseRanges[j + 1])));
+          }
+          else
+          {
+            if (r <= diffLowerCaseRanges[j + 1])
+            {
+              descriptors.add(new CharacterRange(
+                              (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + l - diffLowerCaseRanges[j]),
+                              (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + r - diffLowerCaseRanges[j])));
+              break;
+            }
+
+            descriptors.add(new CharacterRange(
+                           (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + l - diffLowerCaseRanges[j]),
+                           Character.toLowerCase(diffLowerCaseRanges[j + 1])));
+          }
+
+          j += 2;
+          while (r > diffLowerCaseRanges[j])
+          {
+            if (r <= diffLowerCaseRanges[j + 1])
+            {
+              descriptors.add(new CharacterRange(Character.toLowerCase(diffLowerCaseRanges[j]),
+                              (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + r - diffLowerCaseRanges[j])));
+              break;
+            }
+
+            descriptors.add(new CharacterRange(Character.toLowerCase(diffLowerCaseRanges[j]),
+                                                       Character.toLowerCase(diffLowerCaseRanges[j + 1])));
+            j += 2;
+          }
+          break;
+        }
+
+        /* Add ranges for which upper case is different. */
+        j = 0;
+        while (l > diffUpperCaseRanges[j])
+        j += 2;
+
+        if (l < diffUpperCaseRanges[j])
+        {
+          if (r < diffUpperCaseRanges[j])
+            continue;
+
+          if (r <= diffUpperCaseRanges[j + 1])
+          {
+            descriptors.add(new CharacterRange(Character.toUpperCase(diffUpperCaseRanges[j]),
+                 (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + r - diffUpperCaseRanges[j])));
+            continue;
+          }
+
+          descriptors.add(new CharacterRange(Character.toUpperCase(diffUpperCaseRanges[j]),
+                                                     Character.toUpperCase(diffUpperCaseRanges[j + 1])));
         }
         else
         {
-           char l = ((CharacterRange)descriptors.elementAt(i)).left;
-           char r = ((CharacterRange)descriptors.elementAt(i)).right;
-           int j = 0, k = 0;
+          if (r <= diffUpperCaseRanges[j + 1])
+          {
+            descriptors.add(new CharacterRange(
+                            (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + l - diffUpperCaseRanges[j]),
+                            (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + r - diffUpperCaseRanges[j])));
+            continue;
+          }
 
-           /* Add ranges for which lower case is different. */
-           for (;;)
-           {
-              while (l > diffLowerCaseRanges[j])
-                 j += 2;
-
-              if (l < diffLowerCaseRanges[j])
-              {
-                 if (r < diffLowerCaseRanges[j])
-                    break;
-
-                 if (r <= diffLowerCaseRanges[j + 1])
-                 {
-                    descriptors.addElement(new CharacterRange(Character.toLowerCase(diffLowerCaseRanges[j]),
-                                    (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + r - diffLowerCaseRanges[j])));
-                    break;
-                 }
-
-                 descriptors.addElement(new CharacterRange(Character.toLowerCase(diffLowerCaseRanges[j]),
-                                                         Character.toLowerCase(diffLowerCaseRanges[j + 1])));
-              }
-              else
-              {
-                 if (r <= diffLowerCaseRanges[j + 1])
-                 {
-                    descriptors.addElement(new CharacterRange(
-                                    (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + l - diffLowerCaseRanges[j]),
-                                    (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + r - diffLowerCaseRanges[j])));
-                    break;
-                 }
-
-                 descriptors.addElement(new CharacterRange(
-                                      (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + l - diffLowerCaseRanges[j]),
-                                                        Character.toLowerCase(diffLowerCaseRanges[j + 1])));
-              }
-
-              j += 2;
-              while (r > diffLowerCaseRanges[j])
-              {
-                 if (r <= diffLowerCaseRanges[j + 1])
-                 {
-                    descriptors.addElement(new CharacterRange(Character.toLowerCase(diffLowerCaseRanges[j]),
-                                     (char)(Character.toLowerCase(diffLowerCaseRanges[j]) + r - diffLowerCaseRanges[j])));
-                    break;
-                 }
-
-                 descriptors.addElement(new CharacterRange(Character.toLowerCase(diffLowerCaseRanges[j]),
-                                                         Character.toLowerCase(diffLowerCaseRanges[j + 1])));
-                 j += 2;
-              }
-              break;
-           }
-
-           /* Add ranges for which upper case is different. */
-           j = 0; k = 0;
-           while (l > diffUpperCaseRanges[j])
-              j += 2;
-
-           if (l < diffUpperCaseRanges[j])
-           {
-              if (r < diffUpperCaseRanges[j])
-                 continue;
-
-              if (r <= diffUpperCaseRanges[j + 1])
-              {
-                 descriptors.addElement(new CharacterRange(Character.toUpperCase(diffUpperCaseRanges[j]),
-                                 (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + r - diffUpperCaseRanges[j])));
-                 continue;
-              }
-
-              descriptors.addElement(new CharacterRange(Character.toUpperCase(diffUpperCaseRanges[j]),
-                                                      Character.toUpperCase(diffUpperCaseRanges[j + 1])));
-           }
-           else
-           {
-              if (r <= diffUpperCaseRanges[j + 1])
-              {
-                 descriptors.addElement(new CharacterRange(
-                                 (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + l - diffUpperCaseRanges[j]),
-                                 (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + r - diffUpperCaseRanges[j])));
-                 continue;
-              }
-
-              descriptors.addElement(new CharacterRange(
-                                   (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + l - diffUpperCaseRanges[j]),
-                                                     Character.toUpperCase(diffUpperCaseRanges[j + 1])));
-           }
-
-           j += 2;
-           while (r > diffUpperCaseRanges[j])
-           {
-              if (r <= diffUpperCaseRanges[j + 1])
-              {
-                 descriptors.addElement(new CharacterRange(Character.toUpperCase(diffUpperCaseRanges[j]),
-                                  (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + r - diffUpperCaseRanges[j])));
-                 break;
-              }
-
-              descriptors.addElement(new CharacterRange(Character.toUpperCase(diffUpperCaseRanges[j]),
-                                                      Character.toUpperCase(diffUpperCaseRanges[j + 1])));
-              j += 2;
-           }
+          descriptors.add(new CharacterRange(
+                          (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + l - diffUpperCaseRanges[j]),
+                          Character.toUpperCase(diffUpperCaseRanges[j + 1])));
         }
-     }
+
+        j += 2;
+        while (r > diffUpperCaseRanges[j])
+        {
+          if (r <= diffUpperCaseRanges[j + 1])
+          {
+            descriptors.add(new CharacterRange(Character.toUpperCase(diffUpperCaseRanges[j]),
+                            (char)(Character.toUpperCase(diffUpperCaseRanges[j]) + r - diffUpperCaseRanges[j])));
+            break;
+          }
+
+          descriptors.add(new CharacterRange(Character.toUpperCase(diffUpperCaseRanges[j]),
+                                                     Character.toUpperCase(diffUpperCaseRanges[j + 1])));
+          j += 2;
+        }
+      }
+    }
   }
 
   boolean transformed = false;
@@ -301,23 +306,23 @@ static final char[] diffUpperCaseRanges = {
   {
      if (!transformed)
      {
-        if (Options.B("IGNORE_CASE") || ignoreCase)
+        if (Options.getIgnoreCase() || ignoreCase)
         {
 /*
            int i;
            System.out.println("Before:");
            for (i = 0; i < descriptors.size(); i++)
            {
-              if (descriptors.elementAt(i) instanceof SingleCharacter)
+              if (descriptors.get(i) instanceof SingleCharacter)
               {
-                 char c = ((SingleCharacter)descriptors.elementAt(i)).ch;
+                 char c = ((SingleCharacter)descriptors.get(i)).ch;
                  System.out.print((int)c + " ");
               }
               else
               {
-                 char l = ((CharacterRange)descriptors.elementAt(i)).left;
-                 char r = ((CharacterRange)descriptors.elementAt(i)).right;
-      
+                 char l = ((CharacterRange)descriptors.get(i)).left;
+                 char r = ((CharacterRange)descriptors.get(i)).right;
+
                  System.out.print((int)l + "-" + (int)r + " ");
               }
               if ((i + 1) % 6 == 0)
@@ -333,16 +338,16 @@ static final char[] diffUpperCaseRanges = {
            System.out.println("After:");
            for (i = 0; i < descriptors.size(); i++)
            {
-              if (descriptors.elementAt(i) instanceof SingleCharacter)
+              if (descriptors.get(i) instanceof SingleCharacter)
               {
-                 char c = ((SingleCharacter)descriptors.elementAt(i)).ch;
+                 char c = ((SingleCharacter)descriptors.get(i)).ch;
                  System.out.print((int)c + " ");
               }
               else
               {
-                 char l = ((CharacterRange)descriptors.elementAt(i)).left;
-                 char r = ((CharacterRange)descriptors.elementAt(i)).right;
-      
+                 char l = ((CharacterRange)descriptors.get(i)).left;
+                 char r = ((CharacterRange)descriptors.get(i)).right;
+
                  System.out.print((int)l + "-" + (int)r + " ");
               }
               if ((i + 1) % 6 == 0)
@@ -358,6 +363,12 @@ static final char[] diffUpperCaseRanges = {
            SortDescriptors();
      }
 
+     if (descriptors.size() == 0 && !negated_list)
+     {
+        JavaCCErrors.semantic_error(this, "Empty character set is not allowed as it will not match any character.");
+        return new Nfa();
+     }
+
      transformed = true;
      Nfa retVal = new Nfa();
      NfaState startState = retVal.start;
@@ -366,16 +377,16 @@ static final char[] diffUpperCaseRanges = {
 
      for (i = 0; i < descriptors.size(); i++)
      {
-        if (descriptors.elementAt(i) instanceof SingleCharacter)
-           startState.AddChar(((SingleCharacter)descriptors.elementAt(i)).ch);
-        else // if (descriptors.elementAt(i) instanceof CharacterRange)
+        if (descriptors.get(i) instanceof SingleCharacter)
+           startState.AddChar(((SingleCharacter)descriptors.get(i)).ch);
+        else // if (descriptors.get(i) instanceof CharacterRange)
         {
-           CharacterRange cr = (CharacterRange)descriptors.elementAt(i);
+           CharacterRange cr = (CharacterRange)descriptors.get(i);
 
-           if (cr.left == cr.right)
-              startState.AddChar(cr.left);
+           if (cr.getLeft() == cr.getRight())
+              startState.AddChar(cr.getLeft());
            else
-              startState.AddRange(cr.left, cr.right);
+              startState.AddRange(cr.getLeft(), cr.getRight());
         }
      }
 
@@ -386,24 +397,24 @@ static final char[] diffUpperCaseRanges = {
 
   static boolean Overlaps(CharacterRange r1, CharacterRange r2)
   {
-     return (r1.left <= r2.right && r1.right > r2.right);
+     return (r1.getLeft() <= r2.getRight() && r1.getRight() > r2.getRight());
   }
 
   static boolean SubRange(CharacterRange r1, CharacterRange r2)
   {
-     return (r1.left >= r2.left && r1.right <= r2.right);
+     return (r1.getLeft() >= r2.getLeft() && r1.getRight() <= r2.getRight());
   }
 
   static boolean InRange(char c, CharacterRange range)
   {
-     return (c >= range.left && c <= range.right);
+     return (c >= range.getLeft() && c <= range.getRight());
   }
 
   void SortDescriptors()
   {
      int j;
 
-     Vector newDesc = new Vector(descriptors.size());
+     List newDesc = new ArrayList(descriptors.size());
      int cnt = 0;
 
      Outer:
@@ -412,82 +423,81 @@ static final char[] diffUpperCaseRanges = {
         SingleCharacter s;
         CharacterRange range;
 
-        if (descriptors.elementAt(i) instanceof SingleCharacter)
+        if (descriptors.get(i) instanceof SingleCharacter)
         {
-           s = (SingleCharacter)descriptors.elementAt(i);
+           s = (SingleCharacter)descriptors.get(i);
 
            for (j = 0; j < cnt; j++)
            {
-              if (newDesc.elementAt(j) instanceof SingleCharacter)
+              if (newDesc.get(j) instanceof SingleCharacter)
               {
-                 if (((SingleCharacter)newDesc.elementAt(j)).ch > s.ch)
+                 if (((SingleCharacter)newDesc.get(j)).ch > s.ch)
                     break;
-                 else if (((SingleCharacter)newDesc.elementAt(j)).ch == s.ch)
+                 else if (((SingleCharacter)newDesc.get(j)).ch == s.ch)
                     continue Outer;
               }
               else
               {
-                 char l = ((CharacterRange)newDesc.elementAt(j)).left;
+                 char l = ((CharacterRange)newDesc.get(j)).getLeft();
 
-                 if (InRange(s.ch, (CharacterRange)newDesc.elementAt(j)))
+                 if (InRange(s.ch, (CharacterRange)newDesc.get(j)))
                     continue Outer;
                  else if (l > s.ch)
                     break;
               }
            }
 
-           newDesc.insertElementAt(s, j);
+           newDesc.add(j, s);
            cnt++;
         }
         else
         {
-           range = (CharacterRange)descriptors.elementAt(i);
+           range = (CharacterRange)descriptors.get(i);
 
            for (j = 0; j < cnt; j++)
            {
-              if (newDesc.elementAt(j) instanceof SingleCharacter)
+              if (newDesc.get(j) instanceof SingleCharacter)
               {
-                 if (InRange(((SingleCharacter)newDesc.elementAt(j)).ch, range))
+                 if (InRange(((SingleCharacter)newDesc.get(j)).ch, range))
                  {
-                    newDesc.removeElementAt(j--);
+                    newDesc.remove(j--);
                     cnt--;
                  }
-                 else if (((SingleCharacter)newDesc.elementAt(j)).ch > range.right)
+                 else if (((SingleCharacter)newDesc.get(j)).ch > range.getRight())
                     break;
               }
               else
               {
-                 if (SubRange(range, (CharacterRange)newDesc.elementAt(j)))
+                 if (SubRange(range, (CharacterRange)newDesc.get(j)))
                  {
                     continue Outer;
                  }
-                 else if (SubRange((CharacterRange)newDesc.elementAt(j), range))
+                 else if (SubRange((CharacterRange)newDesc.get(j), range))
                  {
-                    newDesc.setElementAt(range, j);
+                    newDesc.set(j, range);
                     continue Outer;
                  }
-                 else if (Overlaps(range,  (CharacterRange)newDesc.elementAt(j)))
+                 else if (Overlaps(range,  (CharacterRange)newDesc.get(j)))
                  {
-                    range.left = (char)(((CharacterRange)newDesc.elementAt(j)).right + 1);
+                    range.setLeft((char)(((CharacterRange)newDesc.get(j)).getRight() + 1));
                  }
-                 else if (Overlaps((CharacterRange)newDesc.elementAt(j), range))
+                 else if (Overlaps((CharacterRange)newDesc.get(j), range))
                  {
                     CharacterRange tmp = range;
-                    ((CharacterRange)newDesc.elementAt(j)).right = (char)(range.left + 1);
-                    range = (CharacterRange)newDesc.elementAt(j);
-                    newDesc.setElementAt(tmp, j);
+                    ((CharacterRange)newDesc.get(j)).setRight((char)(range.getLeft() + 1));
+                    range = (CharacterRange)newDesc.get(j);
+                    newDesc.set(j, tmp);
                  }
-                 else if (((CharacterRange)newDesc.elementAt(j)).left > range.right)
+                 else if (((CharacterRange)newDesc.get(j)).getLeft() > range.getRight())
                     break;
               }
            }
 
-           newDesc.insertElementAt(range, j);
+           newDesc.add(j, range);
            cnt++;
         }
      }
 
-     newDesc.trimToSize();
      descriptors = newDesc;
   }
 
@@ -501,15 +511,15 @@ static final char[] diffUpperCaseRanges = {
      System.out.println("REM. NEG Before:");
      for (i = 0; i < descriptors.size(); i++)
      {
-        if (descriptors.elementAt(i) instanceof SingleCharacter)
+        if (descriptors.get(i) instanceof SingleCharacter)
         {
-           char c = ((SingleCharacter)descriptors.elementAt(i)).ch;
+           char c = ((SingleCharacter)descriptors.get(i)).ch;
            System.out.print((int)c + " ");
         }
         else
         {
-           char l = ((CharacterRange)descriptors.elementAt(i)).left;
-           char r = ((CharacterRange)descriptors.elementAt(i)).right;
+           char l = ((CharacterRange)descriptors.get(i)).left;
+           char r = ((CharacterRange)descriptors.get(i)).right;
 
            System.out.print((int)l + "-" + (int)r + " ");
         }
@@ -517,15 +527,14 @@ static final char[] diffUpperCaseRanges = {
      System.out.println("");
 */
 
-     Vector newDescriptors = new Vector();
+     List newDescriptors = new ArrayList();
      int lastRemoved = -1; // One less than the first valid character.
 
-   OuterLoop:
      for (i = 0; i < descriptors.size(); i++)
      {
-        if (descriptors.elementAt(i) instanceof SingleCharacter)
+        if (descriptors.get(i) instanceof SingleCharacter)
         {
-           char c = ((SingleCharacter)descriptors.elementAt(i)).ch;
+           char c = ((SingleCharacter)descriptors.get(i)).ch;
 
            if (c >= 0 && c <= lastRemoved + 1)
            {
@@ -534,13 +543,13 @@ static final char[] diffUpperCaseRanges = {
            }
 
            //System.out.println("lastRemoved : " + (int)lastRemoved + "; char : "  + (int)c);
-           newDescriptors.addElement(new CharacterRange((char)(lastRemoved + 1),
+           newDescriptors.add(new CharacterRange((char)(lastRemoved + 1),
                                          (char)((lastRemoved = c) - 1)));
         }
         else
         {
-           char l = ((CharacterRange)descriptors.elementAt(i)).left;
-           char r = ((CharacterRange)descriptors.elementAt(i)).right;
+           char l = ((CharacterRange)descriptors.get(i)).getLeft();
+           char r = ((CharacterRange)descriptors.get(i)).getRight();
 
            if (l >= 0 && l <= lastRemoved + 1)
            {
@@ -549,23 +558,23 @@ static final char[] diffUpperCaseRanges = {
            }
 
            //System.out.println("lastRemoved : " + (int)lastRemoved + "; left : "  + l + "; right : " + (int)r);
-           newDescriptors.addElement(new CharacterRange((char)(lastRemoved + 1),
+           newDescriptors.add(new CharacterRange((char)(lastRemoved + 1),
                                             (char)(l - 1)));
            lastRemoved = r;
         }
      }
 
      //System.out.println("lastRem : " + (int)lastRemoved);
-     if (NfaState.unicodeWarningGiven || Options.B("JAVA_UNICODE_ESCAPE"))
+     if (NfaState.unicodeWarningGiven || Options.getJavaUnicodeEscape())
      {
         if (lastRemoved < (char)0xffff)
-           newDescriptors.addElement(new CharacterRange((char)(lastRemoved + 1),
+           newDescriptors.add(new CharacterRange((char)(lastRemoved + 1),
                                      (char)0xffff));
      }
      else
      {
         if (lastRemoved < (char)0xff)
-           newDescriptors.addElement(new CharacterRange((char)(lastRemoved + 1),
+           newDescriptors.add(new CharacterRange((char)(lastRemoved + 1),
                                                                    (char)0xff));
      }
 
@@ -576,15 +585,15 @@ static final char[] diffUpperCaseRanges = {
      System.out.println("REM NEG After:");
      for (i = 0; i < descriptors.size(); i++)
      {
-        if (descriptors.elementAt(i) instanceof SingleCharacter)
+        if (descriptors.get(i) instanceof SingleCharacter)
         {
-           char c = ((SingleCharacter)descriptors.elementAt(i)).ch;
+           char c = ((SingleCharacter)descriptors.get(i)).ch;
            System.out.print((int)c + " ");
         }
         else
         {
-           char l = ((CharacterRange)descriptors.elementAt(i)).left;
-           char r = ((CharacterRange)descriptors.elementAt(i)).right;
+           char l = ((CharacterRange)descriptors.get(i)).left;
+           char r = ((CharacterRange)descriptors.get(i)).right;
 
            System.out.print((int)l + "-" + (int)r + " ");
         }
@@ -599,15 +608,15 @@ static final char[] diffUpperCaseRanges = {
 
   RCharacterList(char c)
   {
-     descriptors = new Vector();
-     descriptors.addElement(new SingleCharacter(c));
-     negated_list = false;
-     ordinal = Integer.MAX_VALUE;
+    descriptors = new ArrayList();
+    descriptors.add(new SingleCharacter(c));
+    negated_list = false;
+    ordinal = Integer.MAX_VALUE;
   }
 
   public boolean CanMatchAnyChar()
   {
-     // Return true only if it is ~[]
-     return negated_list && (descriptors == null || descriptors.size() == 0);
+    // Return true only if it is ~[]
+    return negated_list && (descriptors == null || descriptors.size() == 0);
   }
 }
